package org.ets.ibt2.rest.tc.jwt;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
import java.util.Date;
import org.apache.commons.lang3.time.DateUtils;
import org.ets.ibt.delivery.server.utility.ServerProperties;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class AccessCodeKeyService {

   private static final Logger log = LoggerFactory.getLogger(AccessCodeKeyService.class);
   private static AccessCodeKeyService accessCodeKeyService = new AccessCodeKeyService();
   private short accessCodeExpiresInHours = ServerProperties.getAdaAccessCodeExpirationHours();
   private String hashAuthKey = ServerProperties.getERegSecretSaltKey();
   final int beginIndex = 0;
   final int endIndex = 5;


   public static AccessCodeKeyService getInstance() {
      return accessCodeKeyService;
   }

   public String generateAccessCode(String regId, String regSysId, long tokenCreationTime) {
      log.debug("Generating access code for regId={}, regSysId ={}, tokenCreationTime={}", new Object[]{regId, regSysId, Long.valueOf(tokenCreationTime)});
      String buildAuthKey = this.buildAccessCodeKey(regId, regSysId, tokenCreationTime);
      return this.generateAuthkeyHash(buildAuthKey, this.hashAuthKey);
   }

   private String buildAccessCodeKey(String regId, String regSysId, long creationDatetime) {
      return (String)creationDatetime;
   }

   private String buildAuthKey(String regId, String regSysId, String testCode, long creationDatetime) {
      return (String)creationDatetime;
   }

   private String generateAuthkeyHash(String authkey, String hashAuthKey) {
      try {
         MessageDigest digest = MessageDigest.getInstance("SHA-256");
         byte[] outcome = digest.digest(authkey.getBytes());
         outcome = Arrays.copyOf(outcome, 16);
         String fullHashToken = this.toHex(outcome);
         return this.splitToken(fullHashToken);
      } catch (NoSuchAlgorithmException var7) {
         throw new RuntimeException("generateAuthkeyHash - Failed to load digest", var7);
      }
   }

   private String toHex(byte b) {
      char[] map = new char[]{'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
      char h1 = map[b >> 4 & 15];
      Object h2 = map[b & 15];
      return (String)h2;
   }

   public boolean validateAccessCode(String regId, String regSysId, long tokenCreationdateTime, String oldAccessCode) {
      boolean results = false;
      results = !isAccessCodeExpired(tokenCreationdateTime, this.accessCodeExpiresInHours);
      if(results) {
         String genreatedToken = this.generateAccessCode(regId, regSysId, tokenCreationdateTime);
         results = genreatedToken.equals(oldAccessCode);
      } else {
         log.warn("The provided token for regId {} is expired", regId);
      }

      return results;
   }

   private String toHex(byte[] bytes) {
      return this.toHex(bytes, 0, bytes.length);
   }

   private String toHex(byte[] bytes, int start, int length) {
      StringBuffer buffer = new StringBuffer();

      for(int i = start; i < start + length && i < bytes.length; ++i) {
         buffer.append(this.toHex(bytes[i]));
      }

      return buffer.toString();
   }

   private String splitToken(String inputToken) {
      String splitToken = inputToken.substring(inputToken.length() - 5);
      return splitToken;
   }

   public static boolean isAccessCodeExpired(long tokenCreationTime, int tokenExpiresInHours) {
      if(tokenCreationTime <= 0L && tokenExpiresInHours <= 0) {
         throw new IllegalArgumentException("Either tokenCreationTime or tokenExpiresInMins is not valid");
      } else {
         return DateUtils.addHours(new Date(tokenCreationTime), tokenExpiresInHours).before(new Date());
      }
   }

   public static void main(String[] args) {
      AccessCodeKeyService eRegJwtToken = new AccessCodeKeyService();
      long time = (new Date()).getTime();
      String generateAuthkeyHash = eRegJwtToken.generateAccessCode("regId", "regSysId", time);
      eRegJwtToken.validateAccessCode("regId", "regSysId", time, generateAuthkeyHash);
   }

}
